@@ -20,7 +20,8 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <getopt.h>
+#include <assert.h>
+#include "my_getopt.h"
 #ifdef HAVE_VALGRIND_VALGRIND_H
 #  include <valgrind/valgrind.h>
 #endif
@@ -49,14 +50,18 @@ help (void)
   printf ("Print list of layers.\n"
           "\n");
 #ifdef HAVE_GETOPT_LONG
+  printf ("  -x, --extnames            prints EXTNAMES (r13-r14 only)\n"
+          "                i.e. space instead of _\n");
   printf ("  -f, --flags               prints also flags:\n"
           "                3 chars for: f for frozen, + or - for ON or OFF, l "
           "for locked\n");
-  printf ("      --on                  prints only ON layers\n");
-  printf ("      --help                display this help and exit\n");
+  printf ("  -o, --on                  prints only ON layers\n");
+  printf ("  -h, --help                display this help and exit\n");
   printf ("      --version             output version information and exit\n"
           "\n");
 #else
+  printf ("  -x            prints EXTNAMES (r13-r14 only)\n"
+          "                i.e. space instead of _\n");
   printf ("  -f            prints also flags:\n"
           "                3 chars for: f for frozen, + or - for ON or OFF, l "
           "for locked\n");
@@ -75,26 +80,27 @@ main (int argc, char *argv[])
 {
   int error;
   long i = 1;
-  int flags = 0, on = 0;
+  int flags = 0, on = 0, extnames = 0;
   char *filename_in;
   Dwg_Data dwg;
+  Dwg_Object *obj;
   Dwg_Object_LAYER *layer;
+  Dwg_Object_LAYER_CONTROL *_ctrl;
   int c;
 #ifdef HAVE_GETOPT_LONG
   int option_index = 0;
-  static struct option long_options[] = { { "flags", 0, 0, 'f' },
-                                          { "on", 0, 0, 'o' },
-                                          { "help", 0, 0, 0 },
-                                          { "version", 0, 0, 0 },
-                                          { NULL, 0, NULL, 0 } };
+  static struct option long_options[]
+      = { { "flags", 0, 0, 'f' }, { "extnames", 0, 0, 'x' },
+          { "on", 0, 0, 'o' },    { "help", 0, 0, 0 },
+          { "version", 0, 0, 0 }, { NULL, 0, NULL, 0 } };
 #endif
 
   while
 #ifdef HAVE_GETOPT_LONG
-      ((c = getopt_long (argc, argv, "foh", long_options, &option_index))
+      ((c = getopt_long (argc, argv, "xfoh", long_options, &option_index))
        != -1)
 #else
-      ((c = getopt (argc, argv, "fohi")) != -1)
+      ((c = getopt (argc, argv, "xfohi")) != -1)
 #endif
     {
       if (c == -1)
@@ -112,6 +118,9 @@ main (int argc, char *argv[])
         case 'i':
           return opt_version ();
 #endif
+        case 'x':
+          extnames = 1;
+          break;
         case 'f':
           flags = 1;
           break;
@@ -136,30 +145,63 @@ main (int argc, char *argv[])
   memset (&dwg, 0, sizeof (Dwg_Data));
   error = dwg_read_file (filename_in, &dwg);
   if (error >= DWG_ERR_CRITICAL)
-    fprintf (stderr, "READ ERROR %s: 0x%x\n", filename_in, error);
-
-  for (i = 0; i < dwg.layer_control.num_entries; i++)
     {
-      Dwg_Object *obj = dwg.layer_control.entries[i]->obj;
-      if (obj->type != DWG_TYPE_LAYER) // can be DICTIONARY also
+      fprintf (stderr, "READ ERROR %s: 0x%x\n", filename_in, error);
+      goto done;
+    }
+
+  obj = dwg_get_first_object (&dwg, DWG_TYPE_LAYER_CONTROL);
+  if (!obj)
+    {
+      fprintf (stderr, "No layers found\n");
+      goto done;
+    }
+  _ctrl = obj->tio.object->tio.LAYER_CONTROL;
+  for (i = 0; i < _ctrl->num_entries; i++)
+    {
+      char *name;
+      assert (_ctrl->entries);
+      if (!_ctrl->entries[i]) // NULL BITCODE_H
         continue;
-      layer = dwg.layer_control.entries[i]->obj->tio.object->tio.LAYER;
+      obj = _ctrl->entries[i]->obj;
+      if (!obj || obj->type != DWG_TYPE_LAYER) // can be DICTIONARY also
+        continue;
+      assert (_ctrl->entries[i]->obj->tio.object);
+      layer = _ctrl->entries[i]->obj->tio.object->tio.LAYER;
       if (on && (!layer->on || layer->frozen))
         continue;
       if (flags)
         printf ("%s%s%s\t", layer->frozen ? "f" : " ", layer->on ? "+" : "-",
                 layer->locked ? "l" : " ");
-      // since r2007 unicode, converted to utf-8
-      if (dwg.header.version >= R_2007)
+      if (extnames && dwg.header.from_version >= R_13b1
+          && dwg.header.from_version < R_2000)
         {
-          char *utf8 = bit_convert_TU ((BITCODE_TU)layer->name);
-          printf ("%s\n", utf8);
-          free (utf8);
+          if (!(name = dwg_find_table_extname (&dwg, obj)))
+            name = layer->name;
         }
       else
-        printf ("%s\n", layer->name);
+        name = layer->name;
+      if (!name)
+        {
+          LOG_ERROR ("Invalid layer " FORMAT_RLLx " ignored, empty name",
+                     _ctrl->entries[i]->obj->handle.value)
+        }
+      else
+        {
+          // since r2007 unicode, converted to utf-8
+          if (dwg.header.from_version >= R_2007)
+            {
+              char *utf8 = bit_convert_TU ((BITCODE_TU)name);
+              printf ("%s\n", utf8);
+              free (utf8);
+            }
+          else
+            printf ("%s\n", name);
+        }
+      fflush (stdout);
     }
 
+done:
   // forget about valgrind. really huge DWG's need endlessly here.
   if ((dwg.header.version && dwg.num_objects < 1000)
 #if defined __SANITIZE_ADDRESS__ || __has_feature(address_sanitizer)
