@@ -1,4 +1,4 @@
-// TODO unstable
+// TODO unstable. lots of coverage
 #define DWG_TYPE DWG_TYPE_ASSOCLOFTEDSURFACEACTIONBODY
 #include "common.c"
 
@@ -11,11 +11,12 @@ api_process (dwg_object *obj)
   BITCODE_BL class_version;
 
   Dwg_Version_Type dwg_version = obj->parent->header.version;
-  dwg_obj_assocloftedsurfaceactionbody *_obj = dwg_object_to_ASSOCLOFTEDSURFACEACTIONBODY (obj);
+  dwg_obj_assocloftedsurfaceactionbody *_obj
+      = dwg_object_to_ASSOCLOFTEDSURFACEACTIONBODY (obj);
 
   // ASSOCACTIONBODY
   CHK_ENTITY_TYPE (_obj, ASSOCLOFTEDSURFACEACTIONBODY, aab_version, BL);
-  CHK_ENTITY_MAX  (_obj, ASSOCLOFTEDSURFACEACTIONBODY, aab_version, BL, 2);
+  CHK_ENTITY_MAX (_obj, ASSOCLOFTEDSURFACEACTIONBODY, aab_version, BL, 2);
   CHK_ASSOCPARAMBASEDACTIONBODY (ASSOCLOFTEDSURFACEACTIONBODY);
   // AcDbAssocSurfaceActionBody
   CHK_SUBCLASS_TYPE (_obj->sab, ASSOCSURFACEACTIONBODY, version, BL);
@@ -23,9 +24,9 @@ api_process (dwg_object *obj)
   CHK_SUBCLASS_TYPE (_obj->sab, ASSOCSURFACEACTIONBODY, l2, BL);
   CHK_SUBCLASS_TYPE (_obj->sab, ASSOCSURFACEACTIONBODY, is_semi_ovr, B);
   CHK_SUBCLASS_TYPE (_obj->sab, ASSOCSURFACEACTIONBODY, grip_status, BS);
-  CHK_SUBCLASS_H    (_obj->sab, ASSOCSURFACEACTIONBODY, assocdep);
+  CHK_SUBCLASS_H (_obj->sab, ASSOCSURFACEACTIONBODY, assocdep);
   // AcDbAssocPathBasedSurfaceActionBody
   CHK_ENTITY_TYPE (_obj, ASSOCLOFTEDSURFACEACTIONBODY, pbsab_status, BL);
 
-  CHK_ENTITY_TYPE (_obj, ASSOCLOFTEDSURFACEACTIONBODY, class_version, BL); 
+  CHK_ENTITY_TYPE (_obj, ASSOCLOFTEDSURFACEACTIONBODY, class_version, BL);
 }
