@@ -10,11 +10,12 @@ api_process (dwg_object *obj)
   BITCODE_H *deps;
 
   Dwg_Version_Type dwg_version = obj->parent->header.version;
-  dwg_obj_assocpatchsurfaceactionbody *_obj = dwg_object_to_ASSOCPATCHSURFACEACTIONBODY (obj);
+  dwg_obj_assocpatchsurfaceactionbody *_obj
+      = dwg_object_to_ASSOCPATCHSURFACEACTIONBODY (obj);
 
   // ASSOCACTIONBODY
   CHK_ENTITY_TYPE (_obj, ASSOCPATCHSURFACEACTIONBODY, aab_version, BL);
-  CHK_ENTITY_MAX  (_obj, ASSOCPATCHSURFACEACTIONBODY, aab_version, BL, 2);
+  CHK_ENTITY_MAX (_obj, ASSOCPATCHSURFACEACTIONBODY, aab_version, BL, 2);
   CHK_ASSOCPARAMBASEDACTIONBODY (ASSOCPATCHSURFACEACTIONBODY);
   // AcDbAssocSurfaceActionBody
   CHK_SUBCLASS_TYPE (_obj->sab, ASSOCSURFACEACTIONBODY, version, BL);
@@ -22,7 +23,7 @@ api_process (dwg_object *obj)
   CHK_SUBCLASS_TYPE (_obj->sab, ASSOCSURFACEACTIONBODY, l2, BL);
   CHK_SUBCLASS_TYPE (_obj->sab, ASSOCSURFACEACTIONBODY, is_semi_ovr, B);
   CHK_SUBCLASS_TYPE (_obj->sab, ASSOCSURFACEACTIONBODY, grip_status, BS);
-  CHK_SUBCLASS_H    (_obj->sab, ASSOCSURFACEACTIONBODY, assocdep);
+  CHK_SUBCLASS_H (_obj->sab, ASSOCSURFACEACTIONBODY, assocdep);
   // AcDbAssocPathBasedSurfaceActionBody
   CHK_ENTITY_TYPE (_obj, ASSOCPATCHSURFACEACTIONBODY, pbsab_status, BL);
 }
