@@ -13,7 +13,7 @@ api_process (dwg_object *obj)
   BITCODE_3BD def_label_pt; /* DXF 1011 */
 
   Dwg_Version_Type dwg_version = obj->parent->header.version;
-  //#ifdef DEBUG_CLASSES
+  // #ifdef DEBUG_CLASSES
   dwg_obj_blockpointparameter *_obj = dwg_object_to_BLOCKPOINTPARAMETER (obj);
 
   CHK_EVALEXPR (BLOCKPOINTPARAMETER);
@@ -29,11 +29,10 @@ api_process (dwg_object *obj)
   CHK_ENTITY_3RD (_obj, BLOCKPOINTPARAMETER, def_pt);
   CHK_ENTITY_TYPE (_obj, BLOCKPOINTPARAMETER, num_propinfos, BL);
   CHK_ENTITY_MAX (_obj, BLOCKPOINTPARAMETER, num_propinfos, BL, 2);
-  BLOCKPARAMETER_PropInfo (prop1)
-  BLOCKPARAMETER_PropInfo (prop2)
-  // AcDbBlockPointParameter
-  CHK_ENTITY_UTF8TEXT (_obj, BLOCKPOINTPARAMETER, position_name);
+  BLOCKPARAMETER_PropInfo (prop1) BLOCKPARAMETER_PropInfo (prop2)
+      // AcDbBlockPointParameter
+      CHK_ENTITY_UTF8TEXT (_obj, BLOCKPOINTPARAMETER, position_name);
   CHK_ENTITY_UTF8TEXT (_obj, BLOCKPOINTPARAMETER, position_desc);
   CHK_ENTITY_3RD (_obj, BLOCKPOINTPARAMETER, def_label_pt);
-  //#endif
+  // #endif
 }
