@@ -4,25 +4,14 @@
 #include <stdio.h>
 #include <stddef.h>
 #include <stdarg.h>
-#if defined(BITS_TEST_C) || defined(DECODE_TEST_C)
-#  include "bits.h"
-#endif
-#ifndef DECODE_TEST_C
-  static unsigned int loglevel;
+#if !defined DECODE_TEST_C && !defined ENCODE_TEST_C
+static unsigned int loglevel;
 #  define DWG_LOGLEVEL loglevel
 #  include "logging.h"
 #endif
-#include "config.h"
 #include "common.h"
-
-#if defined(_WIN32) && defined(HAVE_FUNC_ATTRIBUTE_MS_FORMAT) && !defined(__USE_MINGW_ANSI_STDIO)
-#  define ATTRIBUTE_FORMAT(x, y) __attribute__ ((format (ms_printf, x, y)))
-#elif defined HAVE_FUNC_ATTRIBUTE_GNU_FORMAT
-#  define ATTRIBUTE_FORMAT(x, y) __attribute__ ((format (gnu_printf, x, y)))
-#elif defined HAVE_FUNC_ATTRIBUTE_FORMAT
-#  define ATTRIBUTE_FORMAT(x, y) __attribute__ ((format (printf, x, y)))
-#else
-#  define ATTRIBUTE_FORMAT(x, y)
+#if defined(BITS_TEST_C) || defined(DECODE_TEST_C)
+#  include "bits.h"
 #endif
 
 static int num = 0;
@@ -49,9 +38,7 @@ numfailed (void)
   return failed;
 }
 
-static void
-ATTRIBUTE_FORMAT (1, 2)
-ok (const char *fmt, ...)
+static void ATTRIBUTE_FORMAT (1, 2) ok (const char *fmt, ...)
 {
   va_list ap;
 
@@ -70,9 +57,7 @@ pass (void)
   num++;
 }
 
-static void
-ATTRIBUTE_FORMAT (1, 2)
-fail (const char *fmt, ...)
+static void ATTRIBUTE_FORMAT (1, 2) fail (const char *fmt, ...)
 {
   va_list ap;
 
@@ -122,6 +107,7 @@ bitprepare (Bit_Chain *dat, size_t size)
   dat->size = size + 1;
   dat->fh = NULL;
   dat->chain = (unsigned char *)calloc (size + 1, 1);
+  dat->codepage = 30; // ANSI_1252
 }
 
 /*
@@ -148,9 +134,9 @@ Bit_Chain
 strtobt (const char *binarystring)
 {
   Bit_Chain dat;
-  int i;
-  const int length = strlen (binarystring);
-  int size_needed = length / 8;
+  size_t i;
+  const size_t length = strlen (binarystring);
+  unsigned size_needed = (unsigned)(length / 8);
   if (length % 8)
     size_needed++;
 
@@ -175,14 +161,15 @@ strtobt (const char *binarystring)
 
 #endif
 
-// make -s makes it silent, but can be overidden by VERBOSE=1
-int is_make_silent()
+// make -s makes it silent, but can be overridden by VERBOSE=1
+int
+is_make_silent (void)
 {
-  const char *make = getenv("MAKEFLAGS");
+  const char *make = getenv ("MAKEFLAGS");
   if (!make)
-    return 0; 			// not from make: verbose
-  if (strstr (make, "-s") || memBEGINc (make, "s "))  // make check with -s
-    return getenv("VERBOSE") ? 0 : 1;
+    return 0;                                        // not from make: verbose
+  if (strstr (make, "-s") || memBEGINc (make, "s ")) // make check with -s
+    return getenv ("VERBOSE") ? 0 : 1;
   else
-    return 0; 			// make check without -s
+    return 0; // make check without -s
 }
